#Module would create and instance(if no instance ip is provided) and a secondary volume to write SBC image into the volume.

locals {
  instance_ip       = var.INSTANCE_IP != "" ? var.INSTANCE_IP : coalesce(aws_instance.example[0].public_ip, aws_instance.example[0].private_ip)
  instance_id       = var.INSTANCE_IP != "" ? data.aws_instance.myinst[0].id : aws_instance.example[0].id
  availability_zone = var.INSTANCE_IP != "" ? data.aws_instance.myinst[0].availability_zone : aws_instance.example[0].availability_zone
  qcow2_file        = "SBC.qcow2"
  tag_name          = "${terraform.workspace}"
}


#Find the instance if instance IP is provided

data "aws_instance" "myinst" {
  count = var.INSTANCE_IP == "" ? 0 : 1

  filter {
    name   = var.IS_PUBLIC_IP ? "ip-address" : "private-ip-address"
    values = ["${var.INSTANCE_IP}"]
  }
}

# ------------------------------------
# LOOK UP THE LATEST AMAZON LINUX AMI
# ------------------------------------
data "aws_ami" "amazon_linux" {
  most_recent = true
  owners      = ["amazon"]

  filter {
    name   = "architecture"
    values = ["x86_64"]
  }

  filter {
    name   = "block-device-mapping.volume-type"
    values = var.DEFAULT_VOLUME_TYPE
  }

  filter {
    name   = "name"
    values = ["amzn-ami*"]
  }

  filter {
    name   = "virtualization-type"
    values = ["hvm"]
  }
}

# -----------------------------------------------------------
#Provides an EC2 key pair resource.
#A key pair is used to control login access to EC2 instances. 
# -----------------------------------------------------------
resource "aws_key_pair" "sbc_keypair" {
  public_key = file(var.SSH_PUBLIC_KEY)
  count      = var.INSTANCE_IP == "" ? 1 : 0
}

#Manages a single EBS volume.

resource "aws_ebs_volume" "sbc_volume" {
  availability_zone = local.availability_zone
  size              = 20
  tags = {
    Name = "${local.tag_name}"
  }
}

#Provides an EC2 instance resource. This allows instances to be created, 
#updated, and deleted. Instances also support provisioning.

resource "aws_instance" "example" {
  ami                    = data.aws_ami.amazon_linux.id
  instance_type          = "t2.micro"
  subnet_id              = var.AWS_SUBNET_ID
  vpc_security_group_ids = var.AWS_SEC_GRP_ID == "" ? [] : [var.AWS_SEC_GRP_ID]
  key_name               = aws_key_pair.sbc_keypair[0].key_name
  count                  = var.INSTANCE_IP == "" ? 1 : 0

  tags = {
    Name = "${local.tag_name}"
  }
}


#Attach the volume to the AWS Instance.

resource "aws_volume_attachment" "ebs_att" {
  device_name = var.DEV_NAME
  volume_id   = aws_ebs_volume.sbc_volume.id
  instance_id = local.instance_id
}


#Provisioners Without a Resource
#If you need to run provisioners that aren't directly
# associated with a specific resource, 
#you can associate them with a null_resource.

resource "null_resource" "write_sbc_image" {
  depends_on = [aws_volume_attachment.ebs_att]

  triggers = {
    instance_id = local.instance_id
  }

  provisioner "file" {
    source      = var.SBC_QCOW2_FILE
    destination = "/home/${var.INSTANCE_USERNAME}/${local.qcow2_file}"
  }

  provisioner "remote-exec" {
    inline = [
      "sudo yum update -y",
      "sudo yum install qemu-img -y",
      "qemu-img convert  -p ${local.qcow2_file} native_image.raw",
      "sudo dd status=progress if=native_image.raw of=${var.DEV_NAME}",
    ]
  }

  connection {
    host        = local.instance_ip
    type        = "ssh"
    user        = var.INSTANCE_USERNAME
    private_key = file(var.SSH_PRIVATE_KEY)
  }
}
